#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Moduł do zarządzania zdalnymi wdrożeniami i operacjami na urządzeniach.
"""

from infrash.remote.remote_manager import RemoteManager

__all__ = ['RemoteManager']
